#include <pspkernel.h>
#include <pspdisplay.h>
#include <pspdebug.h>
#include <pspgu.h>
#include <pspgum.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

PSP_MODULE_INFO("rollingtest", PSP_MODULE_USER, 1, 1);
static unsigned int __attribute__((aligned(16))) list[262144];
extern unsigned char spr_start[]; // テクスチャ参照
#define BUF_WIDTH (512)
#define SCR_WIDTH (480)
#define SCR_HEIGHT (272)
#define CHR_COUNT 256 // スプライト最大数256を指定

// 描画用Vertex構造体
typedef struct Vertex{
	unsigned short u, v;
	unsigned short color;
	short x, y, z;
}Vertex;
struct Vertex* vertices;

// グローバル変数定義
unsigned long i=0,j=0;	// 汎用
unsigned long m_ulFrameCount=0; // フレームカウント変数
int m_iWide=0,m_iWideBounce=1;

// gu初期化自作関数
void MyInitGu(){
	sceGuInit(); sceGuStart(GU_DIRECT, list);
	sceGuDrawBuffer(GU_PSM_4444, (void*)0, BUF_WIDTH);
	sceGuDispBuffer(SCR_WIDTH, SCR_HEIGHT, (void*)0x44000, BUF_WIDTH);
	sceGuOffset(2048 - (SCR_WIDTH/2), 2048 - (SCR_HEIGHT/2));
	sceGuViewport(2048, 2048, SCR_WIDTH, SCR_HEIGHT);
	sceGuScissor(0, 0, SCR_WIDTH, SCR_HEIGHT);
	sceGuEnable(GU_SCISSOR_TEST); sceGuEnable(GU_TEXTURE_2D);
	// テクスチャデータのセット
	sceGuTexMode(GU_PSM_4444, 0, 0, GU_FALSE);
	sceGuTexImage(0, 256, 256, 256, spr_start);
	sceGuTexFunc(GU_TFX_REPLACE, GU_TCC_RGBA);
	sceGuTexFilter(GU_NEAREST, GU_NEAREST);
	// 透過ルーチン
	sceGuEnable(GU_BLEND);
	sceGuBlendFunc(GU_ADD, GU_SRC_ALPHA, GU_ONE_MINUS_SRC_ALPHA, 0, 0); 
	// 設定完了
	sceGuFinish(); sceGuSync(0, GU_SYNC_FINISH);
	sceDisplayWaitVblankStart(); sceGuDisplay(GU_TRUE);
	// ディスプレイリストからメモリ割り当て(CHR_COUNT利用)
	vertices = (struct Vertex*)sceGuGetMemory(2*CHR_COUNT* sizeof(struct Vertex));
}

// 三角関数テーブル
double cos_table[120],sin_table[120];
void MyGenTriTable(){
	for(i=0;i<120;i++){
		cos_table[i] = cos(i*3.14159/60);
		sin_table[i] = sin(i*3.14159/60);
	}
}

// メインルーチン
int main() {
	MyInitGu();		// gu初期化コール
	MyGenTriTable(); // 三角関数テーブル生成
	// 時計回り用スプライトセット
	for(i=0;i<5;i++){
		vertices[i*2].u = 96; vertices[i*2+1].u = 112;
		vertices[i*2].v = 16; vertices[i*2+1].v = 32;
	}
	// 反時計回り用スプライトセット
	for(i=5;i<10;i++){
		vertices[i*2].u =112; vertices[i*2+1].u = 128;
		vertices[i*2].v = 16; vertices[i*2+1].v = 32;
	}

	while(1){
		sceGuStart(GU_DIRECT,list);
		sceGuClear(GU_COLOR_BUFFER_BIT); // 毎回描画クリア
		m_ulFrameCount++; // フレームカウント変数インクリメント

		// 時計回りの固定幅回転を描画
		for(i=0;i<5;i++){
			vertices[i*2  ].x = 240+(int)(cos_table[(m_ulFrameCount+i*24)%120]*64);
			vertices[i*2+1].x = 240+(int)(cos_table[(m_ulFrameCount+i*24)%120]*64)+16;
			vertices[i*2  ].y = 136+(int)(sin_table[(m_ulFrameCount+i*24)%120]*64);
			vertices[i*2+1].y = 136+(int)(sin_table[(m_ulFrameCount+i*24)%120]*64)+16;
		}
		// 反時計回りの可変幅回転を描画
		for(i=5;i<10;i++){
			vertices[i*2  ].x = 240+(int)(cos_table[119-(30+m_ulFrameCount+i*24)%120]*(48+m_iWide));
			vertices[i*2+1].x = 240+(int)(cos_table[119-(30+m_ulFrameCount+i*24)%120]*(48+m_iWide)+16);
			vertices[i*2  ].y = 136+(int)(sin_table[119-(30+m_ulFrameCount+i*24)%120]*(48+m_iWide));
			vertices[i*2+1].y = 136+(int)(sin_table[119-(30+m_ulFrameCount+i*24)%120]*(48+m_iWide)+16);
		}
		// 可変幅を拡大縮小する
		m_iWide += m_iWideBounce;
		if(m_iWide<0 || m_iWide>60) m_iWideBounce=-m_iWideBounce; // 反転

		// ループ末尾で描画処理を実行
		sceGuDrawArray(GU_SPRITES,GU_TEXTURE_16BIT|GU_COLOR_4444|GU_VERTEX_16BIT|GU_TRANSFORM_2D, CHR_COUNT*2, 0, vertices);
		sceGuFinish(); sceGuSync(0,0); sceDisplayWaitVblankStart(); sceGuSwapBuffers();
	}
	return 0;
}
